/* This is the widget wich draws the outcome of the precision test.
 * For this, it tests whether three points lie on a line or whether
 * they make a left or right turn.
 * 
 * author: Lukas Novosad - novosluk@fel.cvut.cz
 * date: 28/9/2010
 *
 */

#include "orientationwidget.h"

OrientationWidget::OrientationWidget(QWidget *parent)
	: QWidget(parent)
{
	ui.setupUi(this);

	// fix the size of the widget to our needs
	setMinimumSize(256,256);
	setMaximumSize(256,256);

	// set an initial value for the epsilon exponent
	_epsilonExponent = 0;

	// set an initial precision
	_precision = CF;

}

OrientationWidget::~OrientationWidget()
{

}

/*
	This method implements the painting.
	It overrides the abstract definition from the QWidget class.
*/
void OrientationWidget::paintEvent(QPaintEvent * paintEvent)
{
	QPainter painter(this);
	
	switch(_precision)
	{
		case CF:
			{
				Point_CF q(12, 12);
				Point_CF r(24, 24);
				float u = pow(2.0,_epsilonExponent);
				for(int j=0; j<256; j++)
					for(int i=0; i<256; i++)
					{
						Point_CF p(0.5+i*u, 0.5+j*u);
						switch(CGAL::orientation(p,q,r))
						{
							case CGAL::LEFT_TURN:
								painter.setPen(Qt::red);
								painter.drawPoint(i,255-j);
								break;
							case CGAL::RIGHT_TURN:
								painter.setPen(Qt::blue);
								painter.drawPoint(i,255-j);
								break;
							case CGAL::COLLINEAR:
								painter.setPen(Qt::yellow);
								painter.drawPoint(i,255-j);
								break;
						}
					}
				break;
			}

		case CD:
			{
				Point_CD q(12, 12);
				Point_CD r(24, 24);
				double u = pow(2.0,_epsilonExponent);
				for(int j=0; j<256; j++)
					for(int i=0; i<256; i++)
					{
						Point_CD p(0.5+i*u, 0.5+j*u);
						switch(CGAL::orientation(p,q,r))
						{
							case CGAL::LEFT_TURN:
								painter.setPen(Qt::red);
								painter.drawPoint(i,255-j);
								break;
							case CGAL::RIGHT_TURN:
								painter.setPen(Qt::blue);
								painter.drawPoint(i,255-j);
								break;
							case CGAL::COLLINEAR:
								painter.setPen(Qt::yellow);
								painter.drawPoint(i,255-j);
								break;
						}
					}
				break;
			}

		// TODO: add the additional cases for the new point types
		
	}

	// draw the diagonal line in black
	painter.setPen(Qt::black);
	painter.drawLine(0,255,255,0);
	
}

void OrientationWidget::setEpsilonExponent(int value)
{
	_epsilonExponent = value;
	// repaint the widget in order to reflect the updated epsilon value
	update();
}

int OrientationWidget::getEpsilonExponent()
{
	return _epsilonExponent;
}

void OrientationWidget::setPrecision(Precision value)
{
	_precision = value;
	// repaint the widget in order to reflect the updated precision
	update();
}